@@ -4,7 +4,7 @@
 -- Module:      Database.MySQL.Simple
 -- Copyright:   (c) 2011 MailRank, Inc.
 -- License:     BSD3
--- Maintainer:  Bryan O'Sullivan <bos@mailrank.com>
+-- Maintainer:  Paul Rouse <pyr@doynton.org>
 -- Stability:   experimental
 -- Portability: portable
 --
@@ -48,6 +48,7 @@ module Database.MySQL.Simple
     , Connection
     , Query
     , In(..)
+    , Binary(..)
     , Only(..)
     -- ** Exceptions
     , FormatError(fmtMessage, fmtQuery, fmtParams)
@@ -60,6 +61,11 @@ module Database.MySQL.Simple
     -- * Queries that return results
     , query
     , query_
+    -- * Queries that stream results
+    , fold
+    , fold_
+    , forEach
+    , forEach_
     -- * Statements that do not return results
     , execute
     , execute_
@@ -78,19 +84,20 @@ module Database.MySQL.Simple
 import Blaze.ByteString.Builder (Builder, fromByteString, toByteString)
 import Blaze.ByteString.Builder.Char8 (fromChar)
 import Control.Applicative ((<$>), pure)
-import Control.Exception (Exception, onException, throw)
+import Control.Exception (Exception, bracket, onException, throw, throwIO)
 import Control.Monad.Fix (fix)
 import Data.ByteString (ByteString)
 import Data.Int (Int64)
 import Data.List (intersperse)
 import Data.Monoid (mappend, mconcat)
 import Data.Typeable (Typeable)
-import Database.MySQL.Base (Connection)
+import Database.MySQL.Base (Connection, Result)
+import Database.MySQL.Base.Types (Field)
 import Database.MySQL.Simple.Param (Action(..), inQuotes)
 import Database.MySQL.Simple.QueryParams (QueryParams(..))
 import Database.MySQL.Simple.QueryResults (QueryResults(..))
 import Database.MySQL.Simple.Result (ResultError(..))
-import Database.MySQL.Simple.Types (In(..), Only(..), Query(..))
+import Database.MySQL.Simple.Types (Binary(..), In(..), Only(..), Query(..))
 import Text.Regex.PCRE.Light (compile, caseless, match)
 import qualified Data.ByteString.Char8 as B
 import qualified Database.MySQL.Base as Base
@@ -182,13 +189,13 @@ buildQuery conn q template xs = zipParams (split template) <$> mapM sub xs
 execute :: (QueryParams q) => Connection -> Query -> q -> IO Int64
 execute conn template qs = do
   Base.query conn =<< formatQuery conn template qs
-  finishExecute template conn
+  finishExecute conn template
 
 -- | A version of 'execute' that does not perform query substitution.
 execute_ :: Connection -> Query -> IO Int64
 execute_ conn q@(Query stmt) = do
   Base.query conn stmt
-  finishExecute q conn
+  finishExecute conn q
 
 -- | Execute a multi-row @INSERT@, @UPDATE@, or other SQL query that is not
 -- expected to return results.
@@ -200,21 +207,22 @@ executeMany :: (QueryParams q) => Connection -> Query -> [q] -> IO Int64
 executeMany _ _ [] = return 0
 executeMany conn q qs = do
   Base.query conn =<< formatMany conn q qs
-  finishExecute q conn
+  finishExecute conn q
 
-finishExecute :: Query -> Connection -> IO Int64
-finishExecute q conn = do
+finishExecute :: Connection -> Query -> IO Int64
+finishExecute conn q = do
   ncols <- Base.fieldCount (Left conn)
   if ncols /= 0
-    then throw $ QueryError ("execute resulted in " ++ show ncols ++
-                             "-column result") q
+    then throwIO $ QueryError ("execute resulted in " ++ show ncols ++
+                               "-column result") q
     else Base.affectedRows conn
 
 -- | Perform a @SELECT@ or other SQL query that is expected to return
--- results.
+-- results. All results are retrieved and converted before this
+-- function returns.
 --
--- All results are retrieved and converted before this function
--- returns.
+-- When processing large results, this function will consume a lot of
+-- client-side memory.  Consider using 'fold' instead.
 --
 -- Exceptions that may be thrown:
 --
@@ -228,28 +236,102 @@ query :: (QueryParams q, QueryResults r)
          => Connection -> Query -> q -> IO [r]
 query conn template qs = do
   Base.query conn =<< formatQuery conn template qs
-  finishQuery template conn
+  finishQuery conn template
 
 -- | A version of 'query' that does not perform query substitution.
 query_ :: (QueryResults r) => Connection -> Query -> IO [r]
 query_ conn q@(Query que) = do
   Base.query conn que
-  finishQuery q conn
+  finishQuery conn q
 
-finishQuery :: (QueryResults r) => Query -> Connection -> IO [r]
-finishQuery q conn = do
-  r <- Base.storeResult conn
+-- | Perform a @SELECT@ or other SQL query that is expected to return
+-- results. Results are streamed incrementally from the server, and
+-- consumed via a left fold.
+--
+-- The result consumer must be carefully written to execute
+-- quickly. If the consumer is slow, server resources will be tied up,
+-- and other clients may not be able to update the tables from which
+-- the results are being streamed.
+--
+-- When dealing with small results, it may be simpler (and perhaps
+-- faster) to use 'query' instead.
+--
+-- This fold is /not/ strict. The stream consumer is responsible for
+-- forcing the evaluation of its result to avoid space leaks.
+--
+-- Exceptions that may be thrown:
+--
+-- * 'FormatError': the query string could not be formatted correctly.
+--
+-- * 'QueryError': the result contains no columns (i.e. you should be
+--   using 'execute' instead of 'query').
+--
+-- * 'ResultError': result conversion failed.
+fold :: (QueryParams q, QueryResults r) =>
+        Connection
+     -> Query                   -- ^ Query template.
+     -> q                       -- ^ Query parameters.
+     -> a                       -- ^ Initial state for result consumer.
+     -> (a -> r -> IO a)        -- ^ Result consumer.
+     -> IO a
+fold conn template qs z f = do
+  Base.query conn =<< formatQuery conn template qs
+  finishFold conn template z f
+
+-- | A version of 'fold' that does not perform query substitution.
+fold_ :: (QueryResults r) =>
+         Connection
+      -> Query                  -- ^ Query.
+      -> a                      -- ^ Initial state for result consumer.
+      -> (a -> r -> IO a)       -- ^ Result consumer.
+      -> IO a
+fold_ conn q@(Query que) z f = do
+  Base.query conn que
+  finishFold conn q z f
+
+-- | A version of 'fold' that does not transform a state value.
+forEach :: (QueryParams q, QueryResults r) =>
+           Connection
+        -> Query                -- ^ Query template.
+        -> q                    -- ^ Query parameters.
+        -> (r -> IO ())         -- ^ Result consumer.
+        -> IO ()
+forEach conn template qs = fold conn template qs () . const
+{-# INLINE forEach #-}
+
+-- | A version of 'forEach' that does not perform query substitution.
+forEach_ :: (QueryResults r) =>
+            Connection
+         -> Query                -- ^ Query template.
+         -> (r -> IO ())         -- ^ Result consumer.
+         -> IO ()
+forEach_ conn template = fold_ conn template () . const
+{-# INLINE forEach_ #-}
+
+finishQuery :: (QueryResults r) => Connection -> Query -> IO [r]
+finishQuery conn q = withResult (Base.storeResult conn) q $ \r fs ->
+  flip fix [] $ \loop acc -> do
+    row <- Base.fetchRow r
+    case row of
+      [] -> return (reverse acc)
+      _  -> let !c = convertResults fs row
+            in loop (c:acc)
+
+finishFold :: (QueryResults r) =>
+                Connection -> Query -> a -> (a -> r -> IO a) -> IO a
+finishFold conn q z0 f = withResult (Base.useResult conn) q $ \r fs ->
+  flip fix z0 $ \loop z -> do
+    row <- Base.fetchRow r
+    case row of
+      [] -> return z
+      _  -> (f z $! convertResults fs row) >>= loop
+
+withResult :: (IO Result) -> Query -> (Result -> [Field] -> IO a) -> IO a
+withResult fetchResult q act = bracket fetchResult Base.freeResult $ \r -> do
   ncols <- Base.fieldCount (Right r)
   if ncols == 0
-    then throw $ QueryError "query resulted in zero-column result" q
-    else do
-      fs <- Base.fetchFields r
-      flip fix [] $ \loop acc -> do
-        row <- Base.fetchRow r
-        case row of
-          [] -> return (reverse acc)
-          _  -> let !c = convertResults fs row
-                in loop (c:acc)
+    then throwIO $ QueryError "query resulted in zero-column result" q
+    else act r =<< Base.fetchFields r
 
 -- | Execute an action inside a SQL transaction.
 --
@@ -263,7 +345,7 @@ finishQuery q conn = do
 -- 'Base.rollback', then the exception will be rethrown.
 withTransaction :: Connection -> IO a -> IO a
 withTransaction conn act = do
-  execute_ conn "start transaction"
+  _ <- execute_ conn "start transaction"
   r <- act `onException` Base.rollback conn
   Base.commit conn
   return r
@@ -304,9 +386,11 @@ fmtError msg q xs = throw FormatError {
 -- >
 -- > import Database.MySQL.Simple
 -- >
+-- > hello :: IO Int
 -- > hello = do
 -- >   conn <- connect defaultConnectInfo
--- >   query conn "select 2 + 2"
+-- >   [Only i] <- query_ conn "select 2 + 2"
+-- >   return i
 --
 -- A 'Query' value does not represent the actual query that will be
 -- executed, but is a template for constructing the final query.
@@ -367,11 +451,27 @@ fmtError msg q xs = throw FormatError {
 -- The same kind of problem can arise with string literals if you have
 -- the @OverloadedStrings@ language extension enabled.  Again, just
 -- use an explicit type signature if this happens.
+--
+-- Finally, remember that the compiler must be able to infer the type
+-- of a query's /results/ as well as its parameters.  We might like
+-- the following example to work:
+--
+-- > print =<< query_ conn "select 2 + 2"
+--
+-- Unfortunately, while a quick glance tells us that the result type
+-- should be a single row containing a single numeric column, the
+-- compiler has no way to infer what the types are.  We can easily fix
+-- this by providing an explicit type annotation:
+--
+-- > xs <- query_ conn "select 2 + 2"
+-- > print (xs :: [Only Int])
+
 
 -- $only_param
 --
 -- Haskell lacks a single-element tuple type, so if you have just one
--- value you want substituted into a query, what should you do?
+-- value you want substituted into a query or a single-column result,
+-- what should you do?
 --
 -- The obvious approach would appear to be something like this:
 --
@@ -379,13 +479,17 @@ fmtError msg q xs = throw FormatError {
 -- >     ...
 --
 -- Unfortunately, this wreaks havoc with type inference, so we take a
--- different tack. To represent a single value @val@ as a parameter, write
--- a singleton list @[val]@, use 'Just' @val@, or use 'Only' @val@.
+-- different tack. To represent a single value @val@ as a parameter,
+-- write a singleton list @[val]@, use 'Just' @val@, or use 'Only'
+-- @val@.
 --
 -- Here's an example using a singleton list:
 --
 -- > execute conn "insert into users (first_name) values (?)"
 -- >              ["Nuala"]
+--
+-- A row of /n/ query results is represented using an /n/-tuple, so
+-- you should use 'Only' to represent a single-column result.
 
 -- $in
 --
